@@ -17,35 +17,45 @@ limitations under the License.
 import json
 
 from langtrace.trace_attributes import Event, LLMSpanAttributes
-from opentelemetry import baggage
+from langtrace_python_sdk.utils import set_span_attribute
+from opentelemetry import baggage, trace
+from opentelemetry.trace.propagation import set_span_in_context
 from opentelemetry.trace import SpanKind
 from opentelemetry.trace.status import Status, StatusCode
 
+from langtrace_python_sdk.utils.llm import (
+    get_base_url,
+    get_extra_attributes,
+    get_llm_request_attributes,
+    get_llm_url,
+    get_langtrace_attributes,
+    get_span_name,
+    set_event_completion,
+    set_usage_attributes,
+)
 from langtrace_python_sdk.constants.instrumentation.common import (
-    LANGTRACE_ADDITIONAL_SPAN_ATTRIBUTES_KEY, SERVICE_PROVIDERS)
+    LANGTRACE_ADDITIONAL_SPAN_ATTRIBUTES_KEY,
+    SERVICE_PROVIDERS,
+)
 from langtrace_python_sdk.constants.instrumentation.groq import APIS
-from langtrace_python_sdk.utils.llm import (calculate_prompt_tokens,
-                                            estimate_tokens)
+from langtrace_python_sdk.utils.llm import calculate_prompt_tokens, estimate_tokens
+from importlib_metadata import version as v
+
+from langtrace_python_sdk.constants import LANGTRACE_SDK_NAME
+from langtrace.trace_attributes import SpanAttributes
 
 
 def chat_completions_create(original_method, version, tracer):
     """Wrap the `create` method of the `ChatCompletion` class to trace it."""
 
     def traced_method(wrapped, instance, args, kwargs):
-        base_url = (
-            str(instance._client._base_url)
-            if hasattr(instance, "_client") and hasattr(instance._client, "_base_url")
-            else ""
-        )
         service_provider = SERVICE_PROVIDERS["GROQ"]
         # If base url contains perplexity or azure, set the service provider accordingly
-        if "perplexity" in base_url:
+        if "perplexity" in get_base_url(instance):
             service_provider = SERVICE_PROVIDERS["PPLX"]
-        elif "azure" in base_url:
+        elif "azure" in get_base_url(instance):
             service_provider = SERVICE_PROVIDERS["AZURE"]
 
-        extra_attributes = baggage.get_baggage(LANGTRACE_ADDITIONAL_SPAN_ATTRIBUTES_KEY)
-
         # handle tool calls in the kwargs
         llm_prompts = []
         for item in kwargs.get("messages", []):
@@ -75,27 +85,16 @@ def chat_completions_create(original_method, version, tracer):
                 llm_prompts.append(item)
 
         span_attributes = {
-            "langtrace.sdk.name": "langtrace-python-sdk",
-            "langtrace.service.name": service_provider,
-            "langtrace.service.type": "llm",
-            "langtrace.service.version": version,
-            "langtrace.version": "1.0.0",
-            "url.full": base_url,
-            "llm.api": APIS["CHAT_COMPLETION"]["ENDPOINT"],
-            "llm.prompts": json.dumps(llm_prompts),
-            "llm.stream": kwargs.get("stream"),
-            **(extra_attributes if extra_attributes is not None else {}),
+            **get_langtrace_attributes(version, service_provider),
+            **get_llm_request_attributes(kwargs, prompts=llm_prompts),
+            **get_llm_url(instance),
+            SpanAttributes.LLM_PATH: APIS["CHAT_COMPLETION"]["ENDPOINT"],
+            **get_extra_attributes(),
         }
 
         attributes = LLMSpanAttributes(**span_attributes)
 
         tools = []
-        if kwargs.get("temperature") is not None:
-            attributes.llm_temperature = kwargs.get("temperature")
-        if kwargs.get("top_p") is not None:
-            attributes.llm_top_p = kwargs.get("top_p")
-        if kwargs.get("user") is not None:
-            attributes.llm_user = kwargs.get("user")
         if kwargs.get("functions") is not None:
             for function in kwargs.get("functions"):
                 tools.append(json.dumps({"type": "function", "function": function}))
@@ -108,16 +107,19 @@ def chat_completions_create(original_method, version, tracer):
         # with tracer.start_as_current_span(APIS["CHAT_COMPLETION"]["METHOD"],
         #                                   kind=SpanKind.CLIENT) as span:
         span = tracer.start_span(
-            APIS["CHAT_COMPLETION"]["METHOD"], kind=SpanKind.CLIENT
+            name=get_span_name(APIS["CHAT_COMPLETION"]["METHOD"]),
+            kind=SpanKind.CLIENT,
+            context=set_span_in_context(trace.get_current_span()),
         )
         for field, value in attributes.model_dump(by_alias=True).items():
-            if value is not None:
-                span.set_attribute(field, value)
+            set_span_attribute(span, field, value)
         try:
             # Attempt to call the original method
             result = wrapped(*args, **kwargs)
             if kwargs.get("stream") is False or kwargs.get("stream") is None:
-                span.set_attribute("llm.model", result.model)
+                set_span_attribute(
+                    span, SpanAttributes.LLM_RESPONSE_MODEL, result.model
+                )
                 if hasattr(result, "choices") and result.choices is not None:
                     responses = [
                         {
@@ -136,30 +138,27 @@ def chat_completions_create(original_method, version, tracer):
                                 if "content_filter_results" in choice
                                 else {}
                             ),
-                        } for choice in result.choices
+                        }
+                        for choice in result.choices
                     ]
-                    span.set_attribute("llm.responses", json.dumps(responses))
-                else:
-                    responses = []
-                    span.set_attribute("llm.responses", json.dumps(responses))
+                    set_event_completion(span, responses)
+
                 if (
                     hasattr(result, "system_fingerprint")
                     and result.system_fingerprint is not None
                 ):
-                    span.set_attribute(
-                        "llm.system.fingerprint", result.system_fingerprint
+                    set_span_attribute(
+                        span,
+                        SpanAttributes.LLM_SYSTEM_FINGERPRINT,
+                        result.system_fingerprint,
                     )
+
                 # Get the usage
                 if hasattr(result, "usage") and result.usage is not None:
                     usage = result.usage
-                    if usage is not None:
-                        usage_dict = {
-                            "input_tokens": result.usage.prompt_tokens,
-                            "output_tokens": usage.completion_tokens,
-                            "total_tokens": usage.total_tokens,
-                        }
-                        span.set_attribute("llm.token.counts", json.dumps(usage_dict))
-                span.set_status(StatusCode.OK)
+                    set_usage_attributes(span, dict(usage))
+
+                span.set_status(Status(StatusCode.OK))
                 span.end()
                 return result
             else:
@@ -224,57 +223,41 @@ def chat_completions_create(original_method, version, tracer):
                     elif tool_calls:
                         for choice in chunk.choices:
                             tool_call = ""
-                            if (choice.delta and choice.delta.tool_calls is not None):
+                            if choice.delta and choice.delta.tool_calls is not None:
                                 toolcalls = choice.delta.tool_calls
                                 content = []
                                 for tool_call in toolcalls:
-                                    if tool_call and tool_call.function is not None and tool_call.function.arguments is not None:
+                                    if (
+                                        tool_call
+                                        and tool_call.function is not None
+                                        and tool_call.function.arguments is not None
+                                    ):
                                         token_counts = estimate_tokens(
                                             tool_call.function.arguments
                                         )
                                         completion_tokens += token_counts
-                                        content = content + [tool_call.function.arguments]
+                                        content = content + [
+                                            tool_call.function.arguments
+                                        ]
                                     else:
                                         content = content + []
                 else:
                     content = []
-                span.add_event(
-                    Event.STREAM_OUTPUT.value,
-                    {
-                        "response": (
-                            "".join(content)
-                            if len(content) > 0 and content[0] is not None
-                            else ""
-                        )
-                    },
-                )
+
                 result_content.append(content[0] if len(content) > 0 else "")
                 yield chunk
         finally:
             # Finalize span after processing all chunks
             span.add_event(Event.STREAM_END.value)
-            span.set_attribute(
-                "llm.token.counts",
-                json.dumps(
-                    {
-                        "input_tokens": prompt_tokens,
-                        "output_tokens": completion_tokens,
-                        "total_tokens": prompt_tokens + completion_tokens,
-                    }
-                ),
+            set_usage_attributes(
+                span,
+                {"input_tokens": prompt_tokens, "output_tokens": completion_tokens},
             )
-            span.set_attribute(
-                "llm.responses",
-                json.dumps(
-                    [
-                        {
-                            "role": "assistant",
-                            "content": "".join(result_content),
-                        }
-                    ]
-                ),
+            set_event_completion(
+                span, [{"role": "assistant", "content": "".join(result_content)}]
             )
-            span.set_status(StatusCode.OK)
+
+            span.set_status(Status(StatusCode.OK))
             span.end()
 
     # return the wrapped method
@@ -285,20 +268,13 @@ def async_chat_completions_create(original_method, version, tracer):
     """Wrap the `create` method of the `ChatCompletion` class to trace it."""
 
     async def traced_method(wrapped, instance, args, kwargs):
-        base_url = (
-            str(instance._client._base_url)
-            if hasattr(instance, "_client") and hasattr(instance._client, "_base_url")
-            else ""
-        )
         service_provider = SERVICE_PROVIDERS["GROQ"]
         # If base url contains perplexity or azure, set the service provider accordingly
-        if "perplexity" in base_url:
+        if "perplexity" in get_base_url(instance):
             service_provider = SERVICE_PROVIDERS["PPLX"]
-        elif "azure" in base_url:
+        elif "azure" in get_base_url(instance):
             service_provider = SERVICE_PROVIDERS["AZURE"]
 
-        extra_attributes = baggage.get_baggage(LANGTRACE_ADDITIONAL_SPAN_ATTRIBUTES_KEY)
-
         # handle tool calls in the kwargs
         llm_prompts = []
         for item in kwargs.get("messages", []):
@@ -306,21 +282,13 @@ def async_chat_completions_create(original_method, version, tracer):
                 tool_calls = []
                 for tool_call in item.tool_calls:
                     tool_call_dict = {
-                        "id": tool_call.id if hasattr(tool_call, "id") else "",
-                        "type": tool_call.type if hasattr(tool_call, "type") else "",
+                        "id": getattr(tool_call, "id", ""),
+                        "type": getattr(tool_call, "type", ""),
                     }
                     if hasattr(tool_call, "function"):
                         tool_call_dict["function"] = {
-                            "name": (
-                                tool_call.function.name
-                                if hasattr(tool_call.function, "name")
-                                else ""
-                            ),
-                            "arguments": (
-                                tool_call.function.arguments
-                                if hasattr(tool_call.function, "arguments")
-                                else ""
-                            ),
+                            "name": getattr(tool_call.function, "name", ""),
+                            "arguments": getattr(tool_call.function, "arguments", ""),
                         }
                     tool_calls.append(tool_call_dict)
                 llm_prompts.append(tool_calls)
@@ -328,27 +296,17 @@ def async_chat_completions_create(original_method, version, tracer):
                 llm_prompts.append(item)
 
         span_attributes = {
-            "langtrace.sdk.name": "langtrace-python-sdk",
-            "langtrace.service.name": service_provider,
-            "langtrace.service.type": "llm",
-            "langtrace.service.version": version,
-            "langtrace.version": "1.0.0",
-            "url.full": base_url,
-            "llm.api": APIS["CHAT_COMPLETION"]["ENDPOINT"],
-            "llm.prompts": json.dumps(llm_prompts),
-            "llm.stream": kwargs.get("stream"),
-            **(extra_attributes if extra_attributes is not None else {}),
+            **get_langtrace_attributes(version, service_provider),
+            **get_llm_request_attributes(kwargs, prompts=llm_prompts),
+            **get_llm_url(instance),
+            SpanAttributes.LLM_PATH: APIS["CHAT_COMPLETION"]["ENDPOINT"],
+            **get_extra_attributes(),
         }
 
         attributes = LLMSpanAttributes(**span_attributes)
 
         tools = []
-        if kwargs.get("temperature") is not None:
-            attributes.llm_temperature = kwargs.get("temperature")
-        if kwargs.get("top_p") is not None:
-            attributes.llm_top_p = kwargs.get("top_p")
-        if kwargs.get("user") is not None:
-            attributes.llm_user = kwargs.get("user")
+
         if kwargs.get("functions") is not None:
             for function in kwargs.get("functions"):
                 tools.append(json.dumps({"type": "function", "function": function}))
@@ -361,16 +319,17 @@ def async_chat_completions_create(original_method, version, tracer):
         # with tracer.start_as_current_span(APIS["CHAT_COMPLETION"]["METHOD"],
         #                                   kind=SpanKind.CLIENT) as span:
         span = tracer.start_span(
-            APIS["CHAT_COMPLETION"]["METHOD"], kind=SpanKind.CLIENT
+            name=get_span_name(APIS["CHAT_COMPLETION"]["METHOD"]), kind=SpanKind.CLIENT
         )
         for field, value in attributes.model_dump(by_alias=True).items():
-            if value is not None:
-                span.set_attribute(field, value)
+            set_span_attribute(span, field, value)
         try:
             # Attempt to call the original method
             result = await wrapped(*args, **kwargs)
             if kwargs.get("stream") is False or kwargs.get("stream") is None:
-                span.set_attribute("llm.model", result.model)
+                set_span_attribute(
+                    span, SpanAttributes.LLM_RESPONSE_MODEL, result.model
+                )
                 if hasattr(result, "choices") and result.choices is not None:
                     responses = [
                         {
@@ -389,29 +348,28 @@ def async_chat_completions_create(original_method, version, tracer):
                                 if "content_filter_results" in choice
                                 else {}
                             ),
-                        } for choice in result.choices
+                        }
+                        for choice in result.choices
                     ]
-                    span.set_attribute("llm.responses", json.dumps(responses))
-                else:
-                    responses = []
-                    span.set_attribute("llm.responses", json.dumps(responses))
+
+                    set_event_completion(span, responses)
+
                 if (
                     hasattr(result, "system_fingerprint")
                     and result.system_fingerprint is not None
                 ):
-                    span.set_attribute(
-                        "llm.system.fingerprint", result.system_fingerprint
+                    set_span_attribute(
+                        span,
+                        SpanAttributes.LLM_SYSTEM_FINGERPRINT,
+                        result.system_fingerprint,
                     )
+
                 # Get the usage
                 if hasattr(result, "usage") and result.usage is not None:
                     usage = result.usage
                     if usage is not None:
-                        usage_dict = {
-                            "input_tokens": result.usage.prompt_tokens,
-                            "output_tokens": usage.completion_tokens,
-                            "total_tokens": usage.total_tokens,
-                        }
-                        span.set_attribute("llm.token.counts", json.dumps(usage_dict))
+                        set_usage_attributes(span, dict(usage))
+
                 span.set_status(StatusCode.OK)
                 span.end()
                 return result
@@ -454,6 +412,9 @@ def async_chat_completions_create(original_method, version, tracer):
         try:
             async for chunk in result:
                 if hasattr(chunk, "model") and chunk.model is not None:
+                    set_span_attribute(
+                        span, SpanAttributes.LLM_RESPONSE_MODEL, chunk.model
+                    )
                     span.set_attribute("llm.model", chunk.model)
                 if hasattr(chunk, "choices") and chunk.choices is not None:
                     if not function_call and not tool_calls:
@@ -477,24 +438,26 @@ def async_chat_completions_create(original_method, version, tracer):
                     elif tool_calls:
                         for choice in chunk.choices:
                             tool_call = ""
-                            if (choice.delta and choice.delta.tool_calls is not None):
+                            if choice.delta and choice.delta.tool_calls is not None:
                                 toolcalls = choice.delta.tool_calls
                                 content = []
                                 for tool_call in toolcalls:
-                                    if tool_call and tool_call.function is not None and tool_call.function.arguments is not None:
+                                    if (
+                                        tool_call
+                                        and tool_call.function is not None
+                                        and tool_call.function.arguments is not None
+                                    ):
                                         token_counts = estimate_tokens(
                                             tool_call.function.arguments
                                         )
                                         completion_tokens += token_counts
-                                        content = content + [tool_call.function.arguments]
-                                    else:
-                                        content = content + []
+                                        content += [tool_call.function.arguments]
                 else:
                     content = []
                 span.add_event(
-                    Event.STREAM_OUTPUT.value,
+                    Event.RESPONSE.value,
                     {
-                        "response": (
+                        SpanAttributes.LLM_COMPLETIONS: (
                             "".join(content)
                             if len(content) > 0 and content[0] is not None
                             else ""
@@ -506,27 +469,22 @@ def async_chat_completions_create(original_method, version, tracer):
         finally:
             # Finalize span after processing all chunks
             span.add_event(Event.STREAM_END.value)
-            span.set_attribute(
-                "llm.token.counts",
-                json.dumps(
+
+            set_usage_attributes(
+                span,
+                {"input_tokens": prompt_tokens, "output_tokens": completion_tokens},
+            )
+
+            set_event_completion(
+                span,
+                [
                     {
-                        "input_tokens": prompt_tokens,
-                        "output_tokens": completion_tokens,
-                        "total_tokens": prompt_tokens + completion_tokens,
+                        "role": "assistant",
+                        "content": "".join(result_content),
                     }
-                ),
-            )
-            span.set_attribute(
-                "llm.responses",
-                json.dumps(
-                    [
-                        {
-                            "role": "assistant",
-                            "content": "".join(result_content),
-                        }
-                    ]
-                ),
+                ],
             )
+
             span.set_status(StatusCode.OK)
             span.end()
 
